# OnePageAccounting - Journal and general ledger in one page of MySQL

create table journal(
	id int not null auto_increment,
	adate date,
	purpose varchar(80),
	identifier varchar(80),
	primary key (id)
) ;

create table account(
	id int not null auto_increment,
	name varchar(80),
	sign decimal(10,2) default 1.0,
	primary key (id)
) ;

create table entry(
	journal int,
	account int,
	adate date,
	debit decimal(10,2) default 0.0,
	credit decimal(10,2) default 0.0
) ;

delimiter //

create procedure account(aname varchar(80))
	# Create an account
begin
	if locate('Assets', aname)=1 then
		insert into account (name, sign) values (aname, -1.0) ;
	elseif locate('Expenses', aname)=1 then
		insert into account (name, sign) values (aname, -1.0) ;
	elseif locate('Draws', aname)=1 then
		insert into account (name, sign) values (aname, -1.0) ;
	else
		insert into account (name, sign) values (aname,  1.0) ;
	end if ;
end
//

create procedure journal(iDate date, aPurpose varchar(80), anIdentifier varchar(80))
	# Create a journal entry
begin
	insert into journal (adate, purpose, identifier)
	values(iDate, aPurpose, anIdentifier) ;
end
//

create function balanceOn(other date, name varchar(80)) returns decimal(10,2)
	# Return the sum of the balances of all accounts that have name as a prefix
	# on and before the specified date.
begin
	declare temp decimal(10,2) ;
	select sign*(sum(credit)-sum(debit))
		from entry, account
		where entry.account=account.id
		and entry.adate <= other
		and account.name like concat(name, '%')
		into temp ;
		if temp is null then
			return 0.0 ;
		else
			return temp ;
		end if ;
end
//

create function balance(name varchar(80)) returns decimal(10,2)
	# Return the sum of the balances of all accounts with names
	# that have "name" as their prefix on or before today.
begin
	return balanceOn(curdate(), name) ;
end
//

create function lookup(name varchar(80)) returns int
	# Return the named account id or null if it does not exist
begin
	declare id int ;
	declare message varchar(80) ;
	select account.id from account where account.name=name into id ;
	if id is null then
		set message = concat('Account does not exist: ', name) ;
		signal sqlstate value '45000' set message_text = message ;
	end if ;
	return id ;
end
//

create procedure debit(other date, amount decimal(10,2), name varchar(80))
	# Post a debit associated with the previous journal entry.
begin
	declare id int ;
	select lookup(name) into id ;
	insert into entry (journal, adate, account, debit)
		select last_insert_id(), other, id, amount
		from account where account.name=name ;
end
//

create procedure credit(other date, amount decimal(10,2), name varchar(80))
	# Post a credit associated with the previous journal entry.
begin
	declare id int ;
	select lookup(name) into id ;
	insert into entry (journal, adate, account, credit)
		select last_insert_id(), other, id, amount
		from account where account.name=name ;
end
//

create procedure entry(amount decimal(10,2), adebit varchar(80), acredit varchar(80))
	# Post a debit and credit associated with the previous journal entry
begin
	declare edate date ;
	select adate from journal where journal.id=last_insert_id() into edate ;
	call debit( edate, amount, adebit) ;
	call credit(edate, amount, acredit) ;
end
//

create procedure adjust(other date, amount decimal(10,2), adebit varchar(80), acredit varchar(80))
	# Post a debit and credit with an alternative date
begin
	call debit( other, amount, adebit) ;
	call credit(other, amount, acredit) ;
end
//

delimiter ;
